#include "osninput.h"

/*
 * osninput
 *
 * program to manipulate Uctova osnova
 */

void main(int argc, char *argv[])
{
  char commandkey;
  char *acctnum;
  char *analnum;
  char *acctname;

  /* initializion of database */
  initBackend("MDEBS");

  /* primary basic check of arguments */
  if (argc < 2) {
    fprintf(stderr, "Wrong number of arguments.\n");
    finish();
  }
  if (strlen(argv[1]) != 1) {
    fprintf(stderr, "First argument must be a single character.\n");
    finish();
  }

  /* determine what command we are to perform */
  commandkey = *argv[1];
  /*  printf("You entered: '%c'\n", commandkey); */

  /* perform the command */
  switch (commandkey) {

  case ('d'):                    /* DELETE */
    if (argc < 3) {
      fprintf(stderr, "Missing argument(s): need at least 3-digit account number.\n");
      finish();
    }
    if (strlen(argv[2]) != 3) {
      fprintf(stderr, "Error: Account number must have exactly three digits.\n");
      finish();
    } else {
      acctnum = argv[2];
    }
    if (argc >= 4) {
      if (strlen(argv[3]) != 2) {
	fprintf(stderr, "Error: Analytical account number must have exactly two digits.\n");
	finish();
      } else {
	analnum = argv[3];
      }
    } else {
      analnum = "--";
    }
    if (accountQuery(acctnum, analnum) == 0) {
      printf("Nothing to delete!\n");
    } else {
      printf("You have instructed me to delete this record.\n");
      deleteAcct(acctnum, analnum);
    }
    break;

  case ('i'):                   /* INSERT */
    if (argc < 4) {
      fprintf(stderr, "Missing argument(s) - need account number and designation.\n");
      finish();
    }
    acctname = argv[2];
    if (strlen(acctname) > 60) {
      fprintf(stderr, "Account designation too long; please reduce to 60 characters or less.\n");
      finish();
    }
    acctnum = argv[3];
    if (argc >= 5) {
      analnum = argv[4];
    } else {
      analnum = "--";
    }
    if (strlen(acctnum) != 3) {
      fprintf(stderr, "Error: Account number must have exactly three digits.\n");
      finish();
    }
    if (strlen(analnum) != 2) {
      fprintf(stderr, "Error: Analytical account number must have exactly two digits.\n");
      finish();
    }
    if (accountQuery(acctnum, analnum) == 1) {
      printf("Account no. %s %s already exists!\n", acctnum, analnum);
      finish();
    } else {
      printf("Attempting to insert acct. no. %s %s: \"%s\"\n", acctnum, analnum, acctname);
      insertAcct(acctnum, analnum, acctname);
    }
    break;

  case ('q'):                   /* QUERY ONLY */
    if (argc < 3) {
      fprintf(stderr, "Missing second argument - number of account to query.\n");
      finish();
    }
    acctnum = argv[2];
    if (argc >= 4) {
      analnum = argv[3];
    } else {
      analnum = "--";
    }
    accountQuery(acctnum, analnum);
    break;

  default:
    fprintf(stderr, "Not a valid argument.\n");
    fprintf(stderr, "Valid arguments are: <q> Query\n");
    fprintf(stderr, "                     <d> Delete\n");
    fprintf(stderr, "                     <i> Insert\n");
    printf("\007");
  }
  finish();
}

void insertAcct(char *acct, char *anal, char *acctDesig)
{
  char miscbuff[161];
  int resultcode;

  sprintf(miscbuff, "INSERT INTO osnova VALUES ('%s', '%s', '%s');", acct, anal, acctDesig);
  resultcode = runQuery(miscbuff);
  if (resultcode == 2) {
    printf("Insert command successful.\n");
  } else {
    printf("Insert command returned (%d), which is not what I expected.\n", resultcode);
  }
  return;
}

void deleteAcct(char *acct, char *anal)
{
  char miscbuff[81];
  int resultcode;

  printf("Deleting account number %s %s...\n", acct, anal);
  sprintf(miscbuff, "DELETE FROM osnova WHERE acct='%s' and anal='%s';", acct, anal);
  resultcode = runQuery(miscbuff);
  if (resultcode == 2) {
    printf("Delete command successful.\n");
  } else {
    printf("Delete command returned (%d), which is not what I expected.\n", resultcode);
  }
  return;
}

/*
 * accountQuery()
 *
 * Given an account number, displays the account's designation.
 * Bails out on error, returns 0 if account not found, 1 otherwise.
 */
int accountQuery(char *acct, char *anal)
{
  char miscbuff[81];
  int resultcode, noofrecs;

  printf("Querying account number %s %s...\n", acct, anal);
  sprintf(miscbuff, "SELECT * FROM osnova WHERE acct='%s' and anal='%s';", acct, anal);
  resultcode = runQuery(miscbuff);
  /* printf("Query result code was %d.\n", resultcode); */
  if (resultcode == 3) {                /* this is the proper result code */
    noofrecs = numberReturned();
    if (noofrecs > 1) {
      fprintf(stderr, "Error: Corrupted database: Account number must be unique.\n");
      finish();
    } else
      if (noofrecs == 1) {
	/* code to retreive and display the record */
	printf("Account No. %s %s: \"%s\"\n", acct, anal, getField("desig"));
      } else {
        printf("Query returned %d records.\n", noofrecs);
      }
  } else {
    fprintf(stderr, "Error: Query returned improper result code (%d).\n", resultcode);
    finish();
  }
  return noofrecs;
}

void finish()
{
  exit_nicely();
}
